@@ -91,18 +91,9 @@ class Lemmings():
         last_job_id = self.database.get_current_loop_val('job_id')
         last_cpu_time = self.database.get_current_loop_val('start_cpu_time')
 
-
-        # last_job_id = self.database.get_previous_loop_val('job_id')
-        # last_cpu_time = self.database.get_previous_loop_val('start_cpu_time')
-
         new_cpu_time = self.lemmings_job.machine.get_cpu_cost(last_job_id)
         total_cpu_time = last_cpu_time + new_cpu_time
 
-        # self.database.update_previous_loop('end_cpu_time',
-        #                                    total_cpu_time)
-        # self.database.update_current_loop('start_cpu_time',
-        #                                   total_cpu_time)
-       
         self.database.update_current_loop('end_cpu_time',
                                            total_cpu_time)
         #start cpu time should be updated if new loop!
@@ -136,13 +127,28 @@ class Lemmings():
                     return
 
         batch_job = self.lemmings_job.machine.job_template.batch
+
+
         batch_pjob = (self.lemmings_job.machine.pj_template.batch + '\n'
-                      + "lemmings-hpc run "
+                      + "lemmings run "
                       + str(self.lemmings_job.workflow)
                       + " -s post_job"
                       + " --yaml=" + self.lemmings_job.machine.path_yml
                       + '\n')
 
+        # Handle case where user machine path is provided
+        try:
+            # NOTE: consider what to do when --restart, because non-first loop possibility
+            self.lemmings_job.database.get_first_loop_val('used_machine')
+            batch_pjob = batch_pjob.replace(" --yaml=" + self.lemmings_job.machine.path_yml,
+                " --yaml=" + self.lemmings_job.machine.path_yml
+                + " --machine-file="+self.lemmings_job.machine.path_machine)
+        except KeyError:
+            if not self.lemmings_job.database.get_first_loop_val("local_machine_copy"):
+                batch_pjob = batch_pjob.replace(" --yaml=" + self.lemmings_job.machine.path_yml,
+                    " --yaml=" + self.lemmings_job.machine.path_yml
+                    + " --no-machine-copy")
+
         with open(batch_j, 'w') as fout:
             fout.write(batch_job)
         with open(batch_pj, 'w') as fout:
@@ -176,6 +182,7 @@ class Lemmings():
             log_msg += str(table)
             log_msg += "\n\n"
 
+
         if database[chain_name][-1]['safe_stop'] is True:
             log_msg += "Lemmings STOP because using 'safe stop' command\n"
         if 'run_crash' in database[chain_name][-2] and database[chain_name][-2]['run_crash'] is True:
@@ -190,7 +197,7 @@ class Lemmings():
 
 
     def create_replicate_workflows(self):
-        """ Method that controls the generation of multiple workflows for parallel mode 
+        """ Method that controls the generation of multiple workflows for parallel mode
             and submits them according to user settings
 
              Split in smaller functions
@@ -214,9 +221,22 @@ class Lemmings():
 
         # Generation of different workflow folder replicates
         workflows_list = _generate_workflow_replicates(num_workflows, self.lemmings_job, self.database)
-        # Launch workflows from respective folders
-        max_par_wf = _launch_workflows_parallel(workflows_list, self.lemmings_job, self.database)
 
+        # Check if we need to generate a custom machine
+        if self.lemmings_job.machine.custom_machine:
+            # for each WF:
+            #   we need to generate the machine files -> custom_machine.yml
+            #   check if they comply with given SCHEMA -> still to do
+            #   then submit lemmings chains with arguments --machine-file=custom_machine.yml
+            custom_mach_job, custom_mach_pjob = _check_custom_machine_params(self.lemmings_job)
+            _generate_custom_machine_files(self.lemmings_job.machine,
+                                            workflows_list,
+                                            custom_mach_job,
+                                            custom_mach_pjob)
+        # Launch workflows from respective folders
+        max_par_wf = _launch_workflows_parallel(workflows_list,
+                                                self.lemmings_job,
+                                                self.database)
 
         try:
             raise LemmingsStop("Replicate workflows launched according to,"
@@ -224,6 +244,124 @@ class Lemmings():
         except TypeError:
             raise LemmingsStop("All replicate workflows launched")
 
+def _generate_custom_machine_files(machine, wf_list, flag_job_machine, flag_pjob_machine):
+    """ Function that will generate the machine file based on information from the user
+
+        NOTE: consider moving this to a separate tools.py file with other functions
+                or in machine.py or merge in function of Machine class
+
+        Input:
+            :machine: machine object
+            :wf_list: list of type string, contains workflow names
+            :flag_job_machine: boolean, if True, user specified machine job_queue
+                                        if False, job_queue from $LEMMINGS_MACHINE
+            :flag_pjob_machine: boolean, if True, user specified machine pjob_queue
+                                        if False, pjob_queue from $LEMMINGS_MACHINE
+    """
+    from lemmings_hpc.chain.machine import  get_machine_template
+    import yaml
+    # print("IMPORTANT!! ARE THE DICT KEYS READ IN ORDER?")
+    # possibly ok as wf_list is result  of
+    # collections.OrderedDict() -> to check out
+    commands = get_machine_template(machine.user,
+                                    machine.abspath(machine.path_machine),
+                                    machine.job_name, # this doesn't matter for now
+                                    machine.pjob_name, # this doesn't matter for now
+                                    commands_only = True)
+
+    main_dir = os.getcwd()
+    for ii, dir_ in enumerate(wf_list):
+        os.chdir(dir_)
+        dict_info_job = {'wall_time', 'header','core_nb'}
+        dict_info_pj = {'wall_time', 'header'}
+
+        if not flag_job_machine:
+            job_queue = dict(get_machine_template(machine.user,
+                                    machine.abspath(machine.path_machine),
+                                    machine.job_name,
+                                    None,
+                                    commands_only = False)[1]._asdict())
+            job_queue  = dict([(key,job_queue[key]) for key in dict_info_job])
+
+        if not flag_pjob_machine:
+            pjob_queue = dict(get_machine_template(machine.user,
+                                    machine.abspath(machine.path_machine),
+                                    None,
+                                    machine.pjob_name,
+                                    commands_only = False)[2]._asdict())
+            pjob_queue =  dict([(key,pjob_queue[key]) for key in dict_info_pj])
+
+        custom_dict = machine.user.parallel_params['parameter_array'][ii]
+        if flag_job_machine:
+            job_queue =  dict([(key,custom_dict[machine.user.job_queue][0][key])
+                                    for key in dict_info_job])
+
+        if flag_pjob_machine:
+            pjob_queue =  dict([(key,custom_dict[machine.user.pjob_queue][0][key])
+                                    for key in dict_info_pj])
+
+        local_machine = {'commands':dict(commands._asdict()),
+                            'queues':{machine.user.job_queue: job_queue,
+                                    machine.user.pjob_queue: pjob_queue
+                                }
+                            }
+
+        with open('custom_machine.yml', 'w') as yamlout:
+            yaml.dump(local_machine, yamlout)
+        os.chdir(main_dir)
+
+def _check_custom_machine_params(lemmings_job):
+    """ Function that checks that the job_queue / pjob_queue
+        match with an input from the params
+
+        NOTE: 1) consider moving this check to prior to workflow generation
+              2) consider a template for error messages, so uniform rendering within lemmings
+        Input:
+            :lemmings_job: LemmingsJob object
+        Output:
+            :custom_machine_job, custom_machine_pjob: tuple of booleans
+    """
+
+    # replace by list comprehension
+    # def find_setting_in_list_of_dict(param_to_search, dict_to_search):
+    #     for dict_ in dict_to_search:
+    #         if param_to_search not in dict_.keys():
+    #             return False
+    #     return True
+
+    custom_machine_job = False
+    custom_machine_pjob = False
+    if lemmings_job.machine.user.job_queue == 'custom_machine_job':
+        custom_machine_job = True
+
+        setting_in_every_list = [False if  lemmings_job.machine.user.job_queue not in dict_.keys()
+                                else True
+                                for dict_ in lemmings_job.machine.user.parallel_params['parameter_array']]
+        if False in setting_in_every_list:
+            raise LemmingsStop("ERROR: \n"+
+                               "    "+lemmings_job.machine.user.job_queue+
+                               "  keyword not present in every \n"+
+                               "    Workflow, see parameter_array in parallel_params")
+
+    if lemmings_job.machine.user.pjob_queue == 'custom_machine_pjob':
+        custom_machine_pjob = True
+        setting_in_every_list = [False if  lemmings_job.machine.user.pjob_queue not in dict_.keys()
+                                else True
+                                for dict_ in lemmings_job.machine.user.parallel_params['parameter_array']]
+        if False in setting_in_every_list:
+            raise LemmingsStop("ERROR: \n"+
+                               "    "+lemmings_job.machine.user.pjob_queue+
+                               "  keyword not present in every \n"+
+                               "    Workflow, see parameter_array in parallel_params")
+
+    if custom_machine_job is False and  custom_machine_pjob is False:
+        raise LemmingsStop("ERROR: \n"+
+                            "    Please use 'custom_machine_job' and / or 'custom_machine_pjob'\n "+
+                            "    in your {workflow}.yml for the job_queue and pjob_queue, respectively,\n"
+                           +"    if the flag --custom-machine is activated")
+    return custom_machine_job, custom_machine_pjob
+
+
 
 def _perform_lemmings_job_start(lemmings):
     """ Function controls the call sequence associated with the 'start' part of lemmings
@@ -275,7 +413,7 @@ def _perform_lemmings_job_spawn(lemmings):
         _handle_exception(lemmings.lemmings_job,lemmings.database,
                             any_other_exception, lemmings_stop= False)
         safe_stop = True
-    
+
     if safe_stop is False:
         submit_path = lemmings.database.get_current_loop_val('submit_path')
         try:
@@ -339,7 +477,7 @@ def _perform_lemmings_job_post(lemmings):
                 #                                 condition_reached)
                  #--end testing--#
                 lemmings.database.update_current_loop('condition_reached',
-                                                condition_reached)                      
+                                                condition_reached)
                 if condition_reached is None:
                     lemmings.lemmings_job.end_message = "Run crashed"
                 else:
@@ -458,7 +596,7 @@ def _generate_workflow_replicates(workflow_nbs, lemmings_job, database):
         Input:
             :workflow_nbs: int, number of copies to create
             :lemmings_job: lemmings_job object
-            :database:    Database object  
+            :database:    Database object
 
         Output:
             :workflows_list: list of type string containing workflow folders generated
@@ -469,14 +607,22 @@ def _generate_workflow_replicates(workflow_nbs, lemmings_job, database):
     dir_info = os.listdir()
     for ii in np.arange(workflow_nbs):
         tmp_workflow = "WF_%03d" %ii
-        # In case the user specified a name for his workflow
+        # The 'job_prefix' will be used as standard naming for workflow
+        # User can specify an additional name through 'add_farming_suffix'
         try:
-            tmp_workflow += ("_"+
-                lemmings_job.machine.user.parallel_params['parameter_array'][ii]["workflow_name"])
-        except KeyError as excep:   
-            pass
+            if lemmings_job.machine.user.job_prefix is not None:
+                if lemmings_job.machine.user.job_prefix not in ['None', 'none']:
+                    tmp_workflow += "_"+ lemmings_job.machine.user.job_prefix
+        except AttributeError as excep:
+            assert "'GenericDict' object has no attribute 'job_prefix'" in str(excep)
 
         try:
+            tmp_workflow += "_" + lemmings_job.machine.user.parallel_params['parameter_array'][ii]['add_farming_suffix']
+        except KeyError:
+            pass
+
+        try: # TODO: need to handle case when workflows already exist and not overwrite
+             #  -> what should we do then?
             if os.path.isdir(tmp_workflow) and lemmings_job.machine.user.parallel_params["overwrite_dirs"]:
                 shutil.rmtree(tmp_workflow)
             os.mkdir(tmp_workflow)
@@ -493,6 +639,9 @@ def _generate_workflow_replicates(workflow_nbs, lemmings_job, database):
 
         for item in dir_info:
             if os.path.isfile(item):
+                # we need to ensure we do not copy the {workflow}.yml file
+                # as the parallel mode is activated in it.
+                # we will keep it centralised instead in the main folder
                 if item not in [database.db_path.split("./")[-1],
                                     lemmings_job.workflow+".yml"]:
                     shutil.copy(item, tmp_workflow)
@@ -502,22 +651,55 @@ def _generate_workflow_replicates(workflow_nbs, lemmings_job, database):
                         lemmings_job.pathtools.copy_dir(item, tmp_workflow+"/")
 
         workflows_list.append(tmp_workflow)
-        
+
     return workflows_list
 
 def _launch_workflows_parallel(workflows_list, lemmings_job, database):
-    """ Function performing the actual replication of the current work folder structure
-        for separate lemmings calls
+    """ Function performing the actual launch of the lemmings chains in farming mode
 
         Input:
             :workflow_list: list of type string containing names of workflows to consider
             :lemmings_job: lemmings_job object
-            :database:    Database object  
+            :database:    Database object
 
         Output:
             :max_par_wf: None or int with number of max nb of simultaneous workflows to launch
     """
 
+    def _launch_single_chain(job_idx):
+        ''' Function launching normal lemmings through subprocess
+        '''
+
+        try:
+            job_prefix = lemmings_job.machine.user.job_prefix
+            if job_prefix in ['None', 'none']:
+                job_prefix = ""
+        except AttributeError:
+            job_prefix = ""
+
+        try:
+            if job_prefix == "":
+                job_prefix += lemmings_job.machine.user.parallel_params["parameter_array"][job_idx]['add_farming_suffix']
+            else:
+                job_prefix += "_"+lemmings_job.machine.user.parallel_params["parameter_array"][job_idx]['add_farming_suffix']
+        except KeyError:
+            pass
+
+        command = ["lemmings run " + lemmings_job.workflow + " --noverif"
+                                + " --yaml=../"+lemmings_job.workflow+".yml"][0]
+
+        if job_prefix != "":
+            command = command.replace(" --yaml=../"+lemmings_job.workflow+".yml",
+                " --yaml=../"+lemmings_job.workflow+".yml"
+                + " --job-prefix="+str(job_prefix))
+
+        if lemmings_job.machine.custom_machine:
+            command = command.replace(" --yaml=../"+lemmings_job.workflow+".yml",
+                " --yaml=../"+lemmings_job.workflow+".yml"
+                + " --machine-file=custom_machine.yml")
+
+        subprocess.call(command, shell = True)
+
     tmp_path = os.getcwd()
     for ii, workflow in enumerate(workflows_list):
         # go to subdirectory to launch lemmings
@@ -527,8 +709,8 @@ def _launch_workflows_parallel(workflows_list, lemmings_job, database):
         try:
             max_par_wf = lemmings_job.machine.user.parallel_params["max_parallel_workflows"]
             if not ii+1 > max_par_wf:
-                subprocess.call("lemmings-hpc run " + lemmings_job.workflow + " --noverif"
-                                + " --yaml=../"+lemmings_job.workflow+".yml" , shell = True)
+                _launch_single_chain(ii)
+
             # go back to main directory
             os.chdir(tmp_path)
             if not ii+1 > max_par_wf:
@@ -537,13 +719,11 @@ def _launch_workflows_parallel(workflows_list, lemmings_job, database):
                 # case in which we put them on hold for submission at later stage
                 database.update_nested_level_to_loop('parallel_runs',workflow, 'wait')
         except KeyError:
-            subprocess.call("lemmings-hpc run " + lemmings_job.workflow + " --noverif"
-                            + " --yaml=../"+lemmings_job.workflow+".yml" , shell = True)
+            _launch_single_chain(ii)
             os.chdir(tmp_path)
             database.update_nested_level_to_loop('parallel_runs',workflow, 'start')
         except TypeError:
-            subprocess.call("lemmings-hpc run " + lemmings_job.workflow + " --noverif"
-                            + " --yaml=../"+lemmings_job.workflow+".yml" , shell = True)
+            _launch_single_chain(ii)
             os.chdir(tmp_path)
             database.update_nested_level_to_loop('parallel_runs',workflow, 'start')
 
