@@ -4,13 +4,15 @@ import os
 import subprocess
 import numpy as np
 from nob import Nob
-
+import yaml
 
 from lemmings_hpc.chain.path_tool import PathTools
 from lemmings_hpc.chain.database import Database
 
 class LemmingJobBase():
-
+    """ Class containing the different lemmings methods
+        and allowing access to database and other information
+    """
     def __init__(self,
                  workflow,
                  machine,
@@ -66,7 +68,6 @@ class LemmingJobBase():
 
     The following definition of methods allows a single lemmings run to be performed without any other user input
     except for the required .yml file information.
-
     """
     def prior_to_job(self):
         """
@@ -149,6 +150,19 @@ class LemmingJobBase():
                     3) I must update database status
         """
 
+        #-------Parameters----------#
+        use_custom_machine = False
+        #--------------------------#
+
+        # check if a custom machine has been used by the current chain
+        # NOTE: very statefull, but deal with it this way for now
+        try:
+            used_machine = self.database.get_first_loop_val('used_machine')
+            if used_machine.split('/')[-1] == "custom_machine.yml":
+                use_custom_machine = True
+        except KeyError:
+            pass
+
         wf_dir = os.getcwd() # need ton know where I am to update the main database
         wf_current = wf_dir.split('/')[-1]
         # print("Current workflow", wf_current)
@@ -172,20 +186,54 @@ class LemmingJobBase():
         #print(db_info[db_wf_path][:]) # doesn't work directly, need to reread it somehow
 
 
-        # Find next lemmings chain to launch through 'hold' keyword
+        # Find next lemmings chain to launch through 'wait' keyword
         status_list = []
         db_subtree = db_info[self.database.latest_chain_name]
         #print(db_subtree)
-        for key in db_subtree.parallel_runs[:][0].keys():
+        for ii, key in enumerate(db_subtree.parallel_runs[:][0].keys()):
             #print(key)
             #print(db_info[key][:])
             if db_subtree[key][:] == 'wait':
                 print("We found new job to launch ", key)
                 self.database.update_nested_level_to_loop('parallel_runs',key, 'start')
                 os.chdir(key)
+
                 if not debug:
-                    subprocess.call("lemmings-hpc run " + self.workflow + " --noverif"
-                            + " --yaml=../"+self.workflow+".yml" , shell = True)
+                    with open("../"+self.workflow+".yml") as yaml_in:
+                        tmp = yaml.load(yaml_in, Loader=yaml.FullLoader)
+                    try:
+                        job_prefix = tmp['job_prefix']
+                        if job_prefix in ['None', 'none']:
+                            job_prefix = ""
+                    except KeyError:
+                        job_prefix = ""
+
+                    try:
+                        if job_prefix == "":
+                            job_prefix += tmp['parallel_params']['parameter_array'][ii]['add_farming_suffix']
+                        else:
+                            job_prefix += '_' + tmp['parallel_params']['parameter_array'][ii]['add_farming_suffix']
+                    except KeyError:
+                        pass
+
+                    print(job_prefix)
+                    command = ["lemmings run " + self.workflow + " --noverif"
+                            + " --yaml=../"+self.workflow+".yml"][0]
+
+                    if job_prefix != "":
+                        command = command.replace(" --yaml=../"+self.workflow+".yml",
+                            " --yaml=../"+self.workflow+".yml"
+                            + " --job-prefix="+str(job_prefix))
+
+                    if use_custom_machine:
+                        command = command.replace(" --yaml=../"+self.workflow+".yml",
+                                " --yaml=../"+self.workflow+".yml"
+                                + " --machine-file=custom_machine.yml")
+
+                    print(command)
+
+                    # Launch the new workflow
+                    subprocess.call(command, shell = True)
                 os.chdir(base_run_path) # back to main directory
                 if debug:
                     self.database.release_access_to_database()
